@@ -1,5 +1,5 @@
 ;;; Copyright © 2016, 2017, 2018 Roel Janssen <roel@gnu.org>
-;;; Copyright © 2018, 2019, 2020, 2021 Ricardo Wurmus <rekado@elephly.net>
+;;; Copyright © 2018, 2019, 2020, 2021, 2022 Ricardo Wurmus <rekado@elephly.net>
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify it
 ;;; under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 (define-module (gwl workflows)
   #:use-module (gwl config)
   #:use-module (gwl cache)
+  #:use-module (gwl errors)
   #:use-module (gwl oop)
   #:use-module (gwl packages)
   #:use-module (gwl processes)
@@ -33,6 +34,10 @@
                           derivation-output-path))
   #:use-module ((guix ui)
                 #:select (build-notifier))
+  #:use-module ((guix scripts package)
+                #:select (%package-default-options))
+  #:use-module ((guix scripts build)
+                #:select (set-build-options-from-command-line))
   #:use-module ((guix status)
                 #:select (with-status-verbosity))
 
@@ -55,6 +60,8 @@
             workflow-synopsis
             workflow-description
 
+            workflow-kons
+
             print-workflow-record
 
             compute-workflow
@@ -94,14 +101,31 @@
    #:accessor workflow-description
    #:init-keyword #:description
    #:init-value "")
+  (before
+   #:accessor workflow-before
+   #:init-keyword #:before
+   #:init-value (const #true)
+   #:validator procedure?)
+  (after
+   #:accessor workflow-after
+   #:init-keyword #:after
+   #:init-value (const #true)
+   #:validator procedure?)
   (processes
    #:accessor workflow-processes*
    #:init-keyword #:processes
    #:init-value '()
+   #:required? #true
    #:implicit-list? #t
-   #:validator? (lambda (value)
-                  (and (list? value)
-                       (every process? value))))
+   #:validator (lambda (value)
+                 (let loop ((remaining value))
+                   (match remaining
+                     ((head . tail)
+                      (and (if (list? head)
+                               (every process? head)
+                               (process? head))
+                           (loop tail)))
+                     (() #true)))))
   (_restrictions)
   (restrictions
    #:accessor workflow-restrictions
@@ -190,11 +214,13 @@ Use \"processes\" to specify process dependencies.~%"))
 
 (define-immutable-record-type <computed-workflow>
   (make-computed-workflow workflow
+                          name-proc
                           script-proc
                           ordered-processes
                           cache-prefix-proc)
   computed-workflow?
   (workflow           computed-workflow-workflow)           ; <workflow>
+  (name-proc          computed-workflow-name-proc)          ; procedure taking a <process>
   (script-proc        computed-workflow-script-proc)        ; procedure taking a <process>
   (ordered-processes  computed-workflow-ordered-processes)  ; list (of lists) of <process>
   (cache-prefix-proc  computed-workflow-cache-prefix-proc)) ; procedure taking a <process>
@@ -213,20 +239,33 @@ Use \"processes\" to specify process dependencies.~%"))
     (workflow-run-order workflow #:parallel? parallel?))
 
   (define (scripts-by-process)
-    (let ((h (make-hash-table)))
+    (let ((h (make-hash-table))
+          (procedure-table (make-hash-table)))
       (for-each (lambda (process)
-                  (log-event 'debug
-                             (G_ "Computing script for process `~a'~%")
-                             (process-name process))
-                  (hash-set! h process
-                             (process->script process
-                                              #:containerize? containerize?
-                                              #:workflow workflow
-                                              #:input-files
-                                              (lset-intersection
-                                               string=?
-                                               (map second inputs-map-with-extra-files)
-                                               (process-inputs process)))))
+                  (let* ((input-files (lset-intersection
+                                       string=?
+                                       (map second inputs-map-with-extra-files)
+                                       (process-inputs process)))
+                         (key (compile-procedure process)))
+
+                    ;; Skip unnecessary work by looking up existing
+                    ;; scripts by procedure.
+                    (or (and=> (hash-ref procedure-table key)
+                               (lambda (script)
+                                 (hash-set! h process script)))
+
+                        ;; Otherwise compute the script object and
+                        ;; record it for later.
+                        (begin
+                          (log-event 'debug
+                                     (G_ "Computing script for process `~a'~%")
+                                     (process-name process))
+                          (let ((script (process->script process
+                                                         #:containerize? containerize?
+                                                         #:workflow workflow
+                                                         #:input-files input-files)))
+                            (hash-set! h process script)
+                            (hash-set! procedure-table key script))))))
                 (workflow-processes workflow))
       h))
   (define (wrapper-scripts-by-process scripts-table)
@@ -253,7 +292,11 @@ Use \"processes\" to specify process dependencies.~%"))
                                   processes))
            (file-names
             (parameterize ((%guile-for-build (default-guile-derivation)))
+              (log-event 'debug
+                         (G_ "Generating all scripts and their dependencies.~%"))
               (with-status-verbosity (%config 'verbosity)
+                (set-build-options-from-command-line
+                 (inferior-store) %package-default-options)
                 (with-build-handler (build-notifier #:verbosity (%config 'verbosity))
                   (run-with-store (inferior-store)
                     (mlet* %store-monad
@@ -265,9 +308,6 @@ Use \"processes\" to specify process dependencies.~%"))
                                (derivation-output-path output)))
                             (map derivation-outputs drvs))))))))))
       (for-each (lambda (process script-file)
-                  (log-event 'debug
-                             (G_ "Recording script file for process `~a'~%")
-                             (process-name process))
                   (hash-set! h process script-file))
                 processes
                 file-names)
@@ -289,7 +329,24 @@ Use \"processes\" to specify process dependencies.~%"))
                                 ordered-processes
                                 wrapper-script-files-table))
 
+  (define (process->job-name process)
+    "Returns a valid job name for PROCESS."
+    (define sanitize-job-name
+      ;; These characters are not okay for SGE job names.
+      (let ((bad-chars (char-set #\/ #\: #\@ #\\ #\* #\?)))
+        (lambda (name)
+          (string-map
+           (lambda (x)
+             (if (char-set-contains? bad-chars x) #\- x)) name))))
+    (string-append "gwl-"
+                   (sanitize-job-name
+                    (format #false "~a-~a"
+                            (process-full-name process)
+                            (process->hash-string process
+                                                  plain-scripts-table)))))
+
   (make-computed-workflow workflow
+                          process->job-name
                           script-for-process
                           ordered-processes
                           process->cache-prefix))
@@ -379,8 +436,10 @@ contains lists of processes that can be executed in parallel."
                             sequential-execution-order)))
     (or (order-function (workflow-processes workflow)
                         (workflow-restrictions workflow))
-        (error (format (current-error-port)
-                       "error: Cannot determine process execution order.~%")))))
+        (raise (condition
+                (&gwl-error)
+                (&message
+                 (message "Cannot determine process execution order.")))))))
 
 (define (workflow-kons proc)
   "Construct a procedure from the single-argument procedure PROC that
@@ -404,12 +463,7 @@ can be used in a fold over a WORKFLOW's processes."
                            containerize?)
   "Print scripts to be run for WORKFLOW given ENGINE."
   (define computed-workflow
-    (guard (condition
-            ((missing-inputs-condition? condition)
-             (log-event 'error
-                        (G_ "Missing inputs: ~{~%  * ~a~}.~%Provide them with --input=NAME=FILE.~%")
-                        (missing-inputs-files condition))
-             (exit 1)))
+    (begin
       (log-event 'info (G_ "Computing workflow `~a'...~%")
                  (workflow-name workflow))
       (compute-workflow workflow
@@ -438,10 +492,6 @@ return a normalized mapping as a list of two element lists containing
              (list value value)))
        inputs))
 
-(define-condition-type &missing-inputs &condition
-  missing-inputs-condition?
-  (files missing-inputs-files))
-
 (define (prepare-inputs workflow inputs-map)
   "Ensure that all files in the INPUTS-MAP alist exist and are linked
 to the expected locations.  Pick unspecified inputs from the
@@ -502,12 +552,7 @@ to existing files.
 When CONTAINERIZE? is #T build a process script that spawns a
 container."
   (define computed-workflow
-    (guard (condition
-            ((missing-inputs-condition? condition)
-             (log-event 'error
-                        (G_ "Missing inputs: ~{~%  * ~a~}.~%Provide them with --input=NAME=FILE.~%")
-                        (missing-inputs-files condition))
-             (exit 1)))
+    (begin
       (log-event 'info (G_ "Computing workflow `~a'...~%")
                  (workflow-name workflow))
       (compute-workflow workflow
@@ -530,14 +575,60 @@ container."
                         (process-outputs process)))))))
   (define script-for-process
     (computed-workflow-script-proc computed-workflow))
-  (define (run process)
+
+  (define process->job-name
+    (computed-workflow-name-proc computed-workflow))
+
+  (define* (run-local #:key built-script process command cache-prefix job-name)
+    (let* ((status   (run-process-command command))
+           (signal   (status:term-sig status))
+           (exit-val (status:exit-val status)))
+      (when signal
+        (log-event 'error
+                   (G_ "process `~a' termined with signal ~a.~%")
+                   (process-name process)
+                   signal)
+        (exit exit-val))
+      (unless (zero? exit-val)
+        (log-event 'error
+                   (G_ "process `~a' failed with return value ~a.~%")
+                   (process-name process)
+                   exit-val)
+        (exit exit-val)))
+    ;; Wait before generated files are accessed.
+    ;; This may be needed for distributed file
+    ;; systems.
+    (usleep (%cache-delay))
+
+    ;; Abort if declared outputs are missing.
+    (for-each (lambda (output)
+                (let ((canonical-name (if (absolute-file-name? output)
+                                          output
+                                          (string-append (getcwd) "/" output))))
+                  (unless (file-exists? canonical-name)
+                    (log-event 'error
+                               (G_ "process `~a' failed to produce output ~a.~%")
+                               (process-name process)
+                               output)
+                    (exit 1))))
+              (process-outputs process))
+
+    ;; Link files to the cache.
+    (for-each (cut cache! <> cache-prefix)
+              (process-outputs process)))
+
+  (define* (execution-wrapper process #:optional (run run-local))
+    "Use the procedure RUN to launch the script for PROCESS.  This
+procedure wraps the actual runner by adding logging and cache checks."
     (let ((cache-prefix (process->cache-prefix process)))
       (if (cached? process)
           (if dry-run?
-              (log-event 'execute
-                         (G_ "Would skip process \"~a\" (cached at ~a).~%")
-                         (process-name process)
-                         cache-prefix)
+              (begin
+                (log-event 'execute
+                           (G_ "Would skip process \"~a\" (cached at ~a).~%")
+                           (process-name process)
+                           cache-prefix)
+                #false)
               (begin
                 (log-event 'execute
                            (G_ "Skipping process \"~a\" (cached at ~a).~%")
@@ -547,7 +638,8 @@ container."
                 ;; if containerized.  Otherwise link files from
                 ;; cache to expected location.
                 (for-each (cut restore! <> cache-prefix)
-                          (process-outputs process))))
+                          (process-outputs process))
+                #false))
 
           ;; Not cached: execute the process!
           (let* ((built-script (script-for-process process))
@@ -555,39 +647,25 @@ container."
                                         (format #false "'~S'"
                                                 (process->script-arguments process))))))
             (if dry-run?
-                (log-event 'execute
-                           (G_ "Would execute: ~{~a ~}~%") command)
+                (begin
+                  (log-event 'execute
+                             (G_ "Would execute: ~{~a ~}~%") command)
+                  #false)
                 (begin
                   (log-event 'execute
                              (G_ "Executing: ~{~a ~}~%") command)
-                  (let ((retval (apply system* command)))
-                    (unless (zero? retval)
-                      (log-event 'error
-                                 (G_ "error: process `~a' failed with return value ~a.~%")
-                                 (process-name process)
-                                 retval)
-                      (exit retval)))
-                  ;; Wait before generated files are accessed.
-                  ;; This may be needed for distributed file
-                  ;; systems.
-                  (usleep (%cache-delay))
+                  (run #:built-script built-script
+                       #:process process
+                       #:command command
+                       #:cache-prefix cache-prefix
+                       #:job-name (process->job-name process))))))))
 
-                  ;; Abort if declared outputs are missing.
-                  (for-each (lambda (output)
-                              (let ((canonical-name (if (absolute-file-name? output)
-                                                        output
-                                                        (string-append (getcwd) "/" output))))
-                                (unless (file-exists? canonical-name)
-                                  (log-event 'error
-                                             (G_ "process `~a' failed to produce output ~a.~%")
-                                             (process-name process)
-                                             output)
-                                  (exit 1))))
-                            (process-outputs process))
+  ;; Ensure that the environment for the selected process engine is
+  ;; fine before doing any work.
+  ((process-engine-check engine))
 
-                  ;; Link files to the cache.
-                  (for-each (cut cache! <> cache-prefix)
-                            (process-outputs process))))))))
-  (fold (workflow-kons run)
-        '()
-        (computed-workflow-ordered-processes computed-workflow)))
+  ((workflow-before workflow))
+  ((process-engine-run engine)
+   #:wrap execution-wrapper
+   #:processes (computed-workflow-ordered-processes computed-workflow))
+  ((workflow-after workflow)))
